@@ -25,6 +25,8 @@ is_interval = lib.is_interval
 
 is_list_like = lib.is_list_like
 
+is_iterator = lib.is_iterator
+
 
 def is_number(obj) -> bool:
     """
@@ -63,11 +65,10 @@ def is_number(obj) -> bool:
     >>> pd.api.types.is_number("5")
     False
     """
-
     return isinstance(obj, (Number, np.number))
 
 
-def _iterable_not_string(obj) -> bool:
+def iterable_not_string(obj) -> bool:
     """
     Check if the object is an iterable but not a string.
 
@@ -82,51 +83,16 @@ def _iterable_not_string(obj) -> bool:
 
     Examples
     --------
-    >>> _iterable_not_string([1, 2, 3])
+    >>> iterable_not_string([1, 2, 3])
     True
-    >>> _iterable_not_string("foo")
+    >>> iterable_not_string("foo")
     False
-    >>> _iterable_not_string(1)
+    >>> iterable_not_string(1)
     False
     """
-
     return isinstance(obj, abc.Iterable) and not isinstance(obj, str)
 
 
-def is_iterator(obj) -> bool:
-    """
-    Check if the object is an iterator.
-
-    For example, lists are considered iterators
-    but not strings or datetime objects.
-
-    Parameters
-    ----------
-    obj : The object to check
-
-    Returns
-    -------
-    is_iter : bool
-        Whether `obj` is an iterator.
-
-    Examples
-    --------
-    >>> is_iterator([1, 2, 3])
-    True
-    >>> is_iterator(datetime(2017, 1, 1))
-    False
-    >>> is_iterator("foo")
-    False
-    >>> is_iterator(1)
-    False
-    """
-
-    if not hasattr(obj, "__iter__"):
-        return False
-
-    return hasattr(obj, "__next__")
-
-
 def is_file_like(obj) -> bool:
     """
     Check if the object is a file-like object.
@@ -149,13 +115,13 @@ def is_file_like(obj) -> bool:
 
     Examples
     --------
-    >>> buffer(StringIO("data"))
+    >>> import io
+    >>> buffer = io.StringIO("data")
     >>> is_file_like(buffer)
     True
     >>> is_file_like([1, 2, 3])
     False
     """
-
     if not (hasattr(obj, "read") or hasattr(obj, "write")):
         return False
 
@@ -208,7 +174,6 @@ def is_re_compilable(obj) -> bool:
     >>> is_re_compilable(1)
     False
     """
-
     try:
         re.compile(obj)
     except TypeError:
@@ -246,7 +211,6 @@ def is_array_like(obj) -> bool:
     >>> is_array_like(("a", "b"))
     False
     """
-
     return is_list_like(obj) and hasattr(obj, "dtype")
 
 
@@ -343,6 +307,7 @@ def is_named_tuple(obj) -> bool:
 
     Examples
     --------
+    >>> from collections import namedtuple
     >>> Point = namedtuple("Point", ["x", "y"])
     >>> p = Point(1, 2)
     >>>
@@ -351,7 +316,6 @@ def is_named_tuple(obj) -> bool:
     >>> is_named_tuple((1, 2))
     False
     """
-
     return isinstance(obj, tuple) and hasattr(obj, "_fields")
 
 
@@ -371,6 +335,7 @@ def is_hashable(obj) -> bool:
 
     Examples
     --------
+    >>> import collections
     >>> a = ([],)
     >>> isinstance(a, collections.abc.Hashable)
     True
@@ -415,10 +380,45 @@ def is_sequence(obj) -> bool:
     >>> is_sequence(iter(l))
     False
     """
-
     try:
         iter(obj)  # Can iterate over it.
         len(obj)  # Has a length associated with it.
         return not isinstance(obj, (str, bytes))
     except (TypeError, AttributeError):
         return False
+
+
+def is_dataclass(item):
+    """
+    Checks if the object is a data-class instance
+
+    Parameters
+    ----------
+    item : object
+
+    Returns
+    --------
+    is_dataclass : bool
+        True if the item is an instance of a data-class,
+        will return false if you pass the data class itself
+
+    Examples
+    --------
+    >>> from dataclasses import dataclass
+    >>> @dataclass
+    ... class Point:
+    ...     x: int
+    ...     y: int
+
+    >>> is_dataclass(Point)
+    False
+    >>> is_dataclass(Point(0,2))
+    True
+
+    """
+    try:
+        from dataclasses import is_dataclass
+
+        return is_dataclass(item) and not isinstance(item, type)
+    except ImportError:
+        return False
