@@ -1,10 +1,15 @@
-from warnings import catch_warnings, simplefilter
-
 import numpy as np
 import pytest
 
-from pandas import DataFrame, MultiIndex
-from pandas.util import testing as tm
+from pandas import (
+    DataFrame,
+    Float64Index,
+    Int64Index,
+    MultiIndex,
+    date_range,
+    to_datetime,
+)
+import pandas._testing as tm
 
 
 class TestMultiIndexPartial:
@@ -106,11 +111,6 @@ class TestMultiIndexPartial:
         expected = df.loc[("a", "y")][[1, 0]]
         tm.assert_frame_equal(result, expected)
 
-        with catch_warnings(record=True):
-            simplefilter("ignore", FutureWarning)
-            result = df.ix[("a", "y"), [1, 0]]
-        tm.assert_frame_equal(result, expected)
-
         with pytest.raises(KeyError, match=r"\('a', 'foo'\)"):
             df.loc[("a", "foo"), :]
 
@@ -133,7 +133,33 @@ class TestMultiIndexPartial:
 
         # this works...for now
         df["A"].iloc[14] = 5
-        assert df["A"][14] == 5
+        assert df["A"].iloc[14] == 5
+
+    @pytest.mark.parametrize("dtype", [int, float])
+    def test_getitem_intkey_leading_level(
+        self, multiindex_year_month_day_dataframe_random_data, dtype
+    ):
+        # GH#33355 dont fall-back to positional when leading level is int
+        ymd = multiindex_year_month_day_dataframe_random_data
+        levels = ymd.index.levels
+        ymd.index = ymd.index.set_levels([levels[0].astype(dtype)] + levels[1:])
+        ser = ymd["A"]
+        mi = ser.index
+        assert isinstance(mi, MultiIndex)
+        if dtype is int:
+            assert isinstance(mi.levels[0], Int64Index)
+        else:
+            assert isinstance(mi.levels[0], Float64Index)
+
+        assert 14 not in mi.levels[0]
+        assert not mi.levels[0]._should_fallback_to_positional()
+        assert not mi._should_fallback_to_positional()
+
+        with pytest.raises(KeyError, match="14"):
+            ser[14]
+        with pytest.raises(KeyError, match="14"):
+            with tm.assert_produces_warning(FutureWarning):
+                mi.get_value(ser, 14)
 
     # ---------------------------------------------------------------------
     # AMBIGUOUS CASES!
@@ -147,7 +173,7 @@ class TestMultiIndexPartial:
         tm.assert_series_equal(result, expected)
 
         # need to put in some work here
-
+        # FIXME: dont leave commented-out
         # self.ymd.loc[2000, 0] = 0
         # assert (self.ymd.loc[2000]['A'] == 0).all()
 
@@ -189,6 +215,45 @@ class TestMultiIndexPartial:
         expected.loc["bar"] = 0
         tm.assert_series_equal(result, expected)
 
+    @pytest.mark.parametrize(
+        "indexer, exp_idx, exp_values",
+        [
+            (slice("2019-2", None), [to_datetime("2019-02-01")], [2, 3]),
+            (
+                slice(None, "2019-2"),
+                date_range("2019", periods=2, freq="MS"),
+                [0, 1, 2, 3],
+            ),
+        ],
+    )
+    def test_partial_getitem_loc_datetime(self, indexer, exp_idx, exp_values):
+        # GH: 25165
+        date_idx = date_range("2019", periods=2, freq="MS")
+        df = DataFrame(
+            list(range(4)),
+            index=MultiIndex.from_product([date_idx, [0, 1]], names=["x", "y"]),
+        )
+        expected = DataFrame(
+            exp_values,
+            index=MultiIndex.from_product([exp_idx, [0, 1]], names=["x", "y"]),
+        )
+        result = df[indexer]
+        tm.assert_frame_equal(result, expected)
+        result = df.loc[indexer]
+        tm.assert_frame_equal(result, expected)
+
+        result = df.loc(axis=0)[indexer]
+        tm.assert_frame_equal(result, expected)
+
+        result = df.loc[indexer, :]
+        tm.assert_frame_equal(result, expected)
+
+        df2 = df.swaplevel(0, 1).sort_index()
+        expected = expected.swaplevel(0, 1).sort_index()
+
+        result = df2.loc[:, indexer, :]
+        tm.assert_frame_equal(result, expected)
+
 
 def test_loc_getitem_partial_both_axis():
     # gh-12660
