@@ -11,10 +11,6 @@ import pandas._testing as tm
 
 
 class TestToCSV:
-    @pytest.mark.xfail(
-        (3, 6, 5) > sys.version_info,
-        reason=("Python csv library bug (see https://bugs.python.org/issue32255)"),
-    )
     def test_to_csv_with_single_column(self):
         # see gh-18676, https://bugs.python.org/issue32255
         #
@@ -30,7 +26,7 @@ class TestToCSV:
 """
         with tm.ensure_clean("test.csv") as path:
             df1.to_csv(path, header=None, index=None)
-            with open(path, "r") as f:
+            with open(path) as f:
                 assert f.read() == expected1
 
         df2 = DataFrame([1, None])
@@ -40,7 +36,7 @@ class TestToCSV:
 """
         with tm.ensure_clean("test.csv") as path:
             df2.to_csv(path, header=None, index=None)
-            with open(path, "r") as f:
+            with open(path) as f:
                 assert f.read() == expected2
 
     def test_to_csv_defualt_encoding(self):
@@ -62,7 +58,7 @@ class TestToCSV:
 
         with tm.ensure_clean("test.csv") as path:
             df.to_csv(path, quoting=1)  # 1=QUOTE_ALL
-            with open(path, "r") as f:
+            with open(path) as f:
                 assert f.read() == expected
 
         expected = """\
@@ -73,7 +69,7 @@ $1$,$2$
 
         with tm.ensure_clean("test.csv") as path:
             df.to_csv(path, quoting=1, quotechar="$")
-            with open(path, "r") as f:
+            with open(path) as f:
                 assert f.read() == expected
 
         with tm.ensure_clean("test.csv") as path:
@@ -90,7 +86,7 @@ $1$,$2$
 
         with tm.ensure_clean("test.csv") as path:
             df.to_csv(path, quoting=1, doublequote=True)  # QUOTE_ALL
-            with open(path, "r") as f:
+            with open(path) as f:
                 assert f.read() == expected
 
         from _csv import Error
@@ -109,7 +105,7 @@ $1$,$2$
 
         with tm.ensure_clean("test.csv") as path:  # QUOTE_ALL
             df.to_csv(path, quoting=1, doublequote=False, escapechar="\\")
-            with open(path, "r") as f:
+            with open(path) as f:
                 assert f.read() == expected
 
         df = DataFrame({"col": ["a,a", ",bb,"]})
@@ -121,7 +117,7 @@ $1$,$2$
 
         with tm.ensure_clean("test.csv") as path:
             df.to_csv(path, quoting=3, escapechar="\\")  # QUOTE_NONE
-            with open(path, "r") as f:
+            with open(path) as f:
                 assert f.read() == expected
 
     def test_csv_to_string(self):
@@ -154,7 +150,7 @@ $1$,$2$
         )
 
         # see gh-11553: testing if decimal is taken into account for '0.0'
-        df = pd.DataFrame({"a": [0, 1.1], "b": [2.2, 3.3], "c": 1})
+        df = DataFrame({"a": [0, 1.1], "b": [2.2, 3.3], "c": 1})
 
         expected_rows = ["a,b,c", "0^0,2^2,1", "1^1,3^3,1"]
         expected = tm.convert_rows_list_to_csv_str(expected_rows)
@@ -169,7 +165,7 @@ $1$,$2$
     def test_to_csv_float_format(self):
         # testing if float_format is taken into account for the index
         # GH 11553
-        df = pd.DataFrame({"a": [0, 1], "b": [2.2, 3.3], "c": 1})
+        df = DataFrame({"a": [0, 1], "b": [2.2, 3.3], "c": 1})
 
         expected_rows = ["a,b,c", "0,2.20,1", "1,3.30,1"]
         expected = tm.convert_rows_list_to_csv_str(expected_rows)
@@ -338,7 +334,7 @@ $1$,$2$
     def test_to_csv_string_array_ascii(self):
         # GH 10813
         str_array = [{"names": ["foo", "bar"]}, {"names": ["baz", "qux"]}]
-        df = pd.DataFrame(str_array)
+        df = DataFrame(str_array)
         expected_ascii = """\
 ,names
 0,"['foo', 'bar']"
@@ -346,13 +342,13 @@ $1$,$2$
 """
         with tm.ensure_clean("str_test.csv") as path:
             df.to_csv(path, encoding="ascii")
-            with open(path, "r") as f:
+            with open(path) as f:
                 assert f.read() == expected_ascii
 
     def test_to_csv_string_array_utf8(self):
         # GH 10813
         str_array = [{"names": ["foo", "bar"]}, {"names": ["baz", "qux"]}]
-        df = pd.DataFrame(str_array)
+        df = DataFrame(str_array)
         expected_utf8 = """\
 ,names
 0,"['foo', 'bar']"
@@ -360,13 +356,13 @@ $1$,$2$
 """
         with tm.ensure_clean("unicode_test.csv") as path:
             df.to_csv(path, encoding="utf-8")
-            with open(path, "r") as f:
+            with open(path) as f:
                 assert f.read() == expected_utf8
 
     def test_to_csv_string_with_lf(self):
         # GH 20353
         data = {"int": [1, 2, 3], "str_lf": ["abc", "d\nef", "g\nh\n\ni"]}
-        df = pd.DataFrame(data)
+        df = DataFrame(data)
         with tm.ensure_clean("lf_test.csv") as path:
             # case 1: The default line terminator(=os.linesep)(PR 21406)
             os_linesep = os.linesep.encode("utf-8")
@@ -400,7 +396,7 @@ $1$,$2$
     def test_to_csv_string_with_crlf(self):
         # GH 20353
         data = {"int": [1, 2, 3], "str_crlf": ["abc", "d\r\nef", "g\r\nh\r\n\r\ni"]}
-        df = pd.DataFrame(data)
+        df = DataFrame(data)
         with tm.ensure_clean("crlf_test.csv") as path:
             # case 1: The default line terminator(=os.linesep)(PR 21406)
             os_linesep = os.linesep.encode("utf-8")
@@ -438,9 +434,7 @@ $1$,$2$
 
     def test_to_csv_stdout_file(self, capsys):
         # GH 21561
-        df = pd.DataFrame(
-            [["foo", "bar"], ["baz", "qux"]], columns=["name_1", "name_2"]
-        )
+        df = DataFrame([["foo", "bar"], ["baz", "qux"]], columns=["name_1", "name_2"])
         expected_rows = [",name_1,name_2", "0,foo,bar", "1,baz,qux"]
         expected_ascii = tm.convert_rows_list_to_csv_str(expected_rows)
 
@@ -460,7 +454,7 @@ $1$,$2$
     )
     def test_to_csv_write_to_open_file(self):
         # GH 21696
-        df = pd.DataFrame({"a": ["x", "y", "z"]})
+        df = DataFrame({"a": ["x", "y", "z"]})
         expected = """\
 manual header
 x
@@ -471,13 +465,13 @@ z
             with open(path, "w") as f:
                 f.write("manual header\n")
                 df.to_csv(f, header=None, index=None)
-            with open(path, "r") as f:
+            with open(path) as f:
                 assert f.read() == expected
 
     def test_to_csv_write_to_open_file_with_newline_py3(self):
         # see gh-21696
         # see gh-20353
-        df = pd.DataFrame({"a": ["x", "y", "z"]})
+        df = DataFrame({"a": ["x", "y", "z"]})
         expected_rows = ["x", "y", "z"]
         expected = "manual header\n" + tm.convert_rows_list_to_csv_str(expected_rows)
         with tm.ensure_clean("test.txt") as path:
@@ -551,17 +545,17 @@ z
             df.to_csv(
                 path, compression={"method": compression, "archive_name": archive_name}
             )
-            zp = ZipFile(path)
-            expected_arcname = path if archive_name is None else archive_name
-            expected_arcname = os.path.basename(expected_arcname)
-            assert len(zp.filelist) == 1
-            archived_file = os.path.basename(zp.filelist[0].filename)
-            assert archived_file == expected_arcname
+            with ZipFile(path) as zp:
+                expected_arcname = path if archive_name is None else archive_name
+                expected_arcname = os.path.basename(expected_arcname)
+                assert len(zp.filelist) == 1
+                archived_file = os.path.basename(zp.filelist[0].filename)
+                assert archived_file == expected_arcname
 
     @pytest.mark.parametrize("df_new_type", ["Int64"])
     def test_to_csv_na_rep_long_string(self, df_new_type):
         # see gh-25099
-        df = pd.DataFrame({"c": [float("nan")] * 3})
+        df = DataFrame({"c": [float("nan")] * 3})
         df = df.astype(df_new_type)
         expected_rows = ["c", "mynull", "mynull", "mynull"]
         expected = tm.convert_rows_list_to_csv_str(expected_rows)
@@ -607,3 +601,64 @@ z
             ser.to_csv(path, errors=errors)
         # No use in reading back the data as it is not the same anymore
         # due to the error handling
+
+    @pytest.mark.parametrize("mode", ["wb", "w"])
+    def test_to_csv_binary_handle(self, mode):
+        """
+        Binary file objects should work (if 'mode' contains a 'b') or even without
+        it in most cases.
+
+        GH 35058 and GH 19827
+        """
+        df = tm.makeDataFrame()
+        with tm.ensure_clean() as path:
+            with open(path, mode="w+b") as handle:
+                df.to_csv(handle, mode=mode)
+            tm.assert_frame_equal(df, pd.read_csv(path, index_col=0))
+
+    @pytest.mark.parametrize("mode", ["wb", "w"])
+    def test_to_csv_encoding_binary_handle(self, mode):
+        """
+        Binary file objects should honor a specified encoding.
+
+        GH 23854 and GH 13068 with binary handles
+        """
+        # example from GH 23854
+        content = "a, b, 🐟".encode("utf-8-sig")
+        buffer = io.BytesIO(content)
+        df = pd.read_csv(buffer, encoding="utf-8-sig")
+
+        buffer = io.BytesIO()
+        df.to_csv(buffer, mode=mode, encoding="utf-8-sig", index=False)
+        buffer.seek(0)  # tests whether file handle wasn't closed
+        assert buffer.getvalue().startswith(content)
+
+        # example from GH 13068
+        with tm.ensure_clean() as path:
+            with open(path, "w+b") as handle:
+                DataFrame().to_csv(handle, mode=mode, encoding="utf-8-sig")
+
+                handle.seek(0)
+                assert handle.read().startswith(b'\xef\xbb\xbf""')
+
+
+def test_to_csv_iterative_compression_name(compression):
+    # GH 38714
+    df = tm.makeDataFrame()
+    with tm.ensure_clean() as path:
+        df.to_csv(path, compression=compression, chunksize=1)
+        tm.assert_frame_equal(
+            pd.read_csv(path, compression=compression, index_col=0), df
+        )
+
+
+def test_to_csv_iterative_compression_buffer(compression):
+    # GH 38714
+    df = tm.makeDataFrame()
+    with io.BytesIO() as buffer:
+        df.to_csv(buffer, compression=compression, chunksize=1)
+        buffer.seek(0)
+        tm.assert_frame_equal(
+            pd.read_csv(buffer, compression=compression, index_col=0), df
+        )
+        assert not buffer.closed
